function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var findConfig = require('find-config');

var axios = require('axios');

var _require = require('./utils'),
    authenticate = _require.authenticate,
    baseUrl = _require.baseUrl,
    getSplitTranslations = _require.getSplitTranslations,
    getTranslationPackageRoot = _require.getTranslationPackageRoot,
    localeMap = _require.localeMap,
    writeFormattedJsonFile = _require.writeFormattedJsonFile,
    validStatus = _require.validStatus;

var config = findConfig.require('memsource.config.js');

var MEMSOURCE_PROJECT = config.MEMSOURCE_PROJECT;

var _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
    info = _require$setup.info,
    success = _require$setup.success,
    bail = _require$setup.bail;

function syncCompletedJobs() {
  var project = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : MEMSOURCE_PROJECT;
  var token = arguments.length > 1 ? arguments[1] : undefined;
  return axios.get("".concat(baseUrl, "projects/").concat(project), {
    params: {
      token: token
    }
  }).then(
  /*#__PURE__*/
  function () {
    var _ref = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(res) {
      var fileCount, finishedCount, reqs, pageNumber, maxPage, workflowLevelsResponse, workflowLevels;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              fileCount = res.data.progress.totalCount;
              finishedCount = res.data.progress.finishedCount; // We can only resolve one finished translation job at a time.
              // If we have multiple completed jobs, it can lead to conflicts.
              // So we bail if there is more than one

              if (finishedCount > 1) {
                bail("Found ".concat(res.data.progress.finishedCount, " files in ").concat(validStatus, " state. There should only ever be one file. Refusing to download."));
              }

              if (finishedCount === 0) {
                info("No files in ".concat(validStatus, " state. Nothing to do."));
                process.exit(0);
              } // Since the response is paginated, we go through each page and look for DELIVERED files


              reqs = [];
              pageNumber = 0;
              maxPage = Math.ceil(fileCount / 50);
              _context.next = 9;
              return axios.get("".concat(baseUrl, "projects/").concat(project, "/workflowSteps"), {
                params: {
                  token: token
                }
              });

            case 9:
              workflowLevelsResponse = _context.sent;
              workflowLevels = workflowLevelsResponse.data.projectWorkflowSteps.map(function (currentValue) {
                return currentValue.workflowLevel;
              }); // Only get the files that are in validStatus i.e. DELIVERED

              while (pageNumber < maxPage) {
                // Loop through each workflow step to make sure we get a delivered state file in any workflow step
                workflowLevels.forEach(function (workflowLevel) {
                  reqs.push(axios.get("".concat(baseUrl, "projects/").concat(project, "/jobs"), {
                    params: {
                      token: token,
                      pageNumber: pageNumber,
                      workflowLevel: workflowLevel,
                      status: validStatus
                    }
                  }));
                });
                pageNumber++;
              }

              return _context.abrupt("return", Promise.all(reqs));

            case 13:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }()).then(function (results) {
    var reqs = [];
    results.forEach(function (_ref2) {
      var data = _ref2.data;
      data.content.forEach(function (job) {
        var filename = job.filename,
            targetLang = job.targetLang,
            status = job.status;

        if (status === validStatus) {
          reqs.push(axios.get("".concat(baseUrl, "projects/").concat(project, "/jobs/").concat(job.uid, "/targetFile"), {
            params: {
              token: token
            }
          }).then(function (jobData) {
            var phrases = jobData.data;
            info("Job completed for ".concat(filename, ", in target locale: ").concat(targetLang));
            return {
              filename: filename,
              targetLang: targetLang,
              status: status,
              phrases: phrases
            };
          }));
        } else {
          info("File not in ".concat(validStatus, " state. Nothing to do."));
        }
      });
    });
    return Promise.all(reqs);
  });
}

var down =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2() {
    var response, token, jobs, targetLangOutputPath;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return authenticate();

          case 2:
            response = _context2.sent;
            token = response.data.token;
            _context2.next = 6;
            return syncCompletedJobs(MEMSOURCE_PROJECT, token);

          case 6:
            jobs = _context2.sent;
            targetLangOutputPath = "".concat(getTranslationPackageRoot(), "/src/locale");
            jobs.forEach(function (_ref4) {
              var targetLang = _ref4.targetLang,
                  phrases = _ref4.phrases;
              getSplitTranslations(phrases).forEach(function (_ref5) {
                var _ref6 = _slicedToArray(_ref5, 2),
                    filename = _ref6[0],
                    phrases = _ref6[1];

                writeFormattedJsonFile("".concat(targetLangOutputPath, "/").concat(localeMap[targetLang], "/").concat(filename), phrases);
                success('Writing file ' + filename);
              });
            });

          case 9:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function down() {
    return _ref3.apply(this, arguments);
  };
}();

module.exports = down;