function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var findConfig = require('find-config');

var util = require('util');

var exec = util.promisify(require('child_process').exec);

var axios = require('axios');

var _require = require('./utils'),
    baseUrl = _require.baseUrl,
    authenticate = _require.authenticate,
    buildModuleCatalogBuilder = _require.buildModuleCatalogBuilder,
    getCombinedTranslations = _require.getCombinedTranslations,
    getTranslationsMap = _require.getTranslationsMap;

var config = findConfig.require('memsource.config.js');

var MEMSOURCE_PROJECT = config.MEMSOURCE_PROJECT,
    MEMSOURCE_FILE_IMPORT_SETTINGS_ID = config.MEMSOURCE_FILE_IMPORT_SETTINGS_ID;

var _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
    bail = _require$setup.bail,
    success = _require$setup.success,
    warn = _require$setup.warn;

var targetLangs = ['de_de'];
var importSettings = {
  uid: MEMSOURCE_FILE_IMPORT_SETTINGS_ID
};
var memsourceSettings = {
  targetLangs: targetLangs,
  importSettings: importSettings
};
/**
 * createMemsourceJob
 * Given a translation JSON object, send a curl request to create
 * a job in Memsource
 *
 * @param {*} token - Memsource token
 * @param {*} translation - translation JSON object
 * @param {*} remoteFileName - file name to be used in Memsource
 */

var createMemsourceJob =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(token, translation, remoteFileName) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return axios.post("".concat(baseUrl, "projects/").concat(MEMSOURCE_PROJECT, "/jobs"), translation, {
              params: {
                token: token
              },
              headers: {
                'content-type': 'application/octet-stream',
                'content-disposition': "filename*=UTF-8''".concat(remoteFileName),
                memsource: JSON.stringify(memsourceSettings)
              }
            });

          case 3:
            success('Uploaded file - ' + remoteFileName);
            _context.next = 9;
            break;

          case 6:
            _context.prev = 6;
            _context.t0 = _context["catch"](0);
            bail('Failed to upload file - ' + remoteFileName + '\n' + _context.t0);

          case 9:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this, [[0, 6]]);
  }));

  return function createMemsourceJob(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * push
 * Determines whether to upload all translations files at once or
 * upload them one by one
 *
 * @param {string} token - Memsource token
 * @param {string} translations - translations JSON object
 * @returns {Promise} promise that resolves/rejects with succeeded/failed file name
 */


var push =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(token, translations, isCombined) {
    var _arr, _i, fileName, remoteFileName, _remoteFileName;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (isCombined) {
              _context2.next = 13;
              break;
            }

            _arr = Object.keys(translations);
            _i = 0;

          case 3:
            if (!(_i < _arr.length)) {
              _context2.next = 11;
              break;
            }

            fileName = _arr[_i];
            remoteFileName = "".concat(fileName.split('.')[0], "-").concat(new Date().toISOString(), ".json");
            _context2.next = 8;
            return createMemsourceJob(token, translations[fileName], remoteFileName);

          case 8:
            _i++;
            _context2.next = 3;
            break;

          case 11:
            _context2.next = 16;
            break;

          case 13:
            _remoteFileName = "combined-translations-".concat(new Date().toISOString(), ".json");
            _context2.next = 16;
            return createMemsourceJob(token, translations, _remoteFileName);

          case 16:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function push(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * up
 * This function gathers all the translations files and passes
 * them to `push(...)` for uploading to Memsource.
 *
 * @param {Object} cmd - object of command line arguments passed to memsource up
 */


var up =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(cmd) {
    var response, token, moduleCatalogBuilderScriptPath, _ref4, stderr, stdout, translations, combinedTranslations;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return authenticate();

          case 2:
            response = _context3.sent;
            token = response.data.token; // Write module catalogs as standard JSON catalogs

            _context3.prev = 4;
            _context3.next = 7;
            return buildModuleCatalogBuilder();

          case 7:
            moduleCatalogBuilderScriptPath = _context3.sent;
            _context3.next = 10;
            return exec("node ".concat(moduleCatalogBuilderScriptPath));

          case 10:
            _ref4 = _context3.sent;
            stderr = _ref4.stderr;
            stdout = _ref4.stdout;

            if (stdout) {
              success(stdout);
            }

            if (!stderr) {
              _context3.next = 16;
              break;
            }

            throw new Error(stderr);

          case 16:
            _context3.next = 22;
            break;

          case 18:
            _context3.prev = 18;
            _context3.t0 = _context3["catch"](4);
            // Since module catalogs are experimental, let's just warn and continue
            warn('Error creating module catalogs');
            warn(_context3.t0);

          case 22:
            // Combine individual json files into one combined.json
            translations = getTranslationsMap();
            combinedTranslations = getCombinedTranslations(translations); // Push the combined translations to memsource

            push(token, combinedTranslations, cmd.combined === true);

          case 25:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3, this, [[4, 18]]);
  }));

  return function up(_x7) {
    return _ref3.apply(this, arguments);
  };
}();

module.exports = up;