"use strict";

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
    bail = _require$setup.bail,
    info = _require$setup.info,
    error = _require$setup.error;

var _require = require('./utils'),
    getTranslationsMap = _require.getTranslationsMap,
    getNumOfLinks = _require.getNumOfLinks,
    getLocales = _require.getLocales;
/**
 * getLinkErrors
 * Checks for valid markdown link syntax given a JSON file.
 *
 * @param {Array} [{String} filename, {Object} textInFile]
 * @return {Array} errors found
 */


var getLinkErrors = function getLinkErrors(_ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      filename = _ref2[0],
      textInFile = _ref2[1];

  var errors = [];
  var leftSquare = 0,
      rightSquare = 0,
      leftParen = 0,
      rightParen = 0;
  Object.keys(textInFile).forEach(function (t) {
    var markdown = textInFile[t];

    if (typeof markdown === 'string') {
      // create a mapped array of the number of occurences of each type of
      // bracket in the markdown
      var _map = ['\\[', '\\]', '\\(', '\\)'].map(function (b) {
        return (markdown.match(new RegExp(b, 'g')) || []).length;
      });

      var _map2 = _slicedToArray(_map, 4);

      leftSquare = _map2[0];
      rightSquare = _map2[1];
      leftParen = _map2[2];
      rightParen = _map2[3];

      if (leftSquare !== rightSquare || leftParen !== rightParen) {
        // markdown such as 'A) ...' is accepted
        if (markdown[1] !== ')') {
          errors.push({
            errorName: 'Missing Brackets or Parentheses',
            filename: filename,
            translationKey: t,
            markdown: markdown
          });
        }
      } else if ((leftSquare > 0 || rightSquare > 0) && (leftParen > 0 || rightParen > 0)) {
        // there are brackets but the regex can't find any links
        if (getNumOfLinks(markdown) === 0) {
          errors.push({
            errorName: 'Invalid Markdown Link Syntax',
            filename: filename,
            translationKey: t,
            markdown: markdown
          });
        }
      }
    }
  });
  return errors;
};
/**
 * showLinkErrors
 * Gets the translations as JSON objects from every file
 * for each locale and calls getLinkErrors. If there are
 * any errors in the file, the error is logged.
 */


var showLinkErrors = function showLinkErrors() {
  getLocales().forEach(function (locale) {
    var translations = getTranslationsMap(locale);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = translations[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var fileObj = _step.value;
        var errors = getLinkErrors(fileObj);

        if (errors.length) {
          errors.forEach(function (_ref3) {
            var errorName = _ref3.errorName,
                filename = _ref3.filename,
                translationKey = _ref3.translationKey,
                markdown = _ref3.markdown;
            error(errorName.toUpperCase());
            info("".concat(filename, " | ").concat(translationKey));
            info("\n".concat(markdown, "\n"));
          });
          bail('There are translation errors.');
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  });
};

module.exports = {
  showLinkErrors: showLinkErrors,
  getLinkErrors: getLinkErrors
};