const axios = require('axios');
const { baseUrl, authenticate } = require('./utils');
const { bail, success, info } = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
});
const findConfig = require('find-config');
const config = findConfig.require('memsource.config.js');
const { MEMSOURCE_FILE_IMPORT_SETTINGS } = config;

/**
 * createFileImportSettings
 * Creates settings based on the MEMSOURCE_FILE_IMPORT_SETINGS key
 *
 */
const createFileImportSettings = async () => {
  // Bail if we can't find file settings config key
  if (typeof MEMSOURCE_FILE_IMPORT_SETTINGS === undefined) {
    bail('Missing MEMSOURCE_FILE_IMPORT_SETTINGS in memsource.config.js');
  }
  const response = await authenticate();
  const token = response.data.token;
  try {
    const settingsResponse = await axios.post(
      `${baseUrl}importSettings/`,
      MEMSOURCE_FILE_IMPORT_SETTINGS,
      {
        params: { token }
      }
    );
    info(JSON.stringify(settingsResponse.data, null, 2));
    success('Created import settings successfully');
  } catch (e) {
    bail('Failed to create file import settings' + '\n' + e);
  }
};

module.exports = createFileImportSettings;
